﻿using System;
using System.Drawing;
using DevExpress.ExpressApp.ConditionalAppearance;
using DevExpress.ExpressApp.Security.Strategy;
using DevExpress.Persistent.Base;
using DevExpress.Xpo;
using DevExpress.Xpo.Metadata;
using K12MIS.Module.BusinessObjects.Enums;

namespace K12MIS.Module.BusinessObjects
{
    [DefaultClassOptions]
    public class Employee : SecuritySystemUser
    {
        //public Employee() : base()
        //{
        //    // This constructor is used when an object is loaded from a persistent storage.
        //    // Do not place any code here.
        //}

        public Employee(Session session) : base(session)
        {
            // This constructor is used when an object is loaded from a persistent storage.
            // Do not place any code here.
        }

        public override void AfterConstruction()
        {
            base.AfterConstruction();
            // Place here your initialization code.
        }


        string lastName;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string LastName
        {
            get => lastName;
            set => SetPropertyValue(nameof(LastName), ref lastName, value);
        }

        string firstName;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string FirstName
        {
            get => firstName;
            set => SetPropertyValue(nameof(FirstName), ref firstName, value);
        }

        string middleName;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string MiddleName
        {
            get => middleName;
            set => SetPropertyValue(nameof(MiddleName), ref middleName, value);
        }

        DateTime dateOfBirth;
        public DateTime DateOfBirth
        {
            get => dateOfBirth;
            set => SetPropertyValue(nameof(DateOfBirth), ref dateOfBirth, value);
        }

        [NonPersistent]
        public int Age
        {
            get
            {
                if (DateOfBirth == DateTime.MinValue)
                    return 0;

                return (int)(DateTime.Today.Subtract(DateOfBirth).TotalDays / 365);
            }
        }

        private Gender _Gender;
        public Gender Gender
        {
            get
            {
                return _Gender;
            }
            set
            {
                SetPropertyValue("Gender", ref _Gender, value);
            }
        }

        string address;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Address
        {
            get => address;
            set => SetPropertyValue(nameof(Address), ref address, value);
        }

        string contactNo;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string ContactNo
        {
            get => contactNo;
            set => SetPropertyValue(nameof(ContactNo), ref contactNo, value);
        }

        string email;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Email
        {
            get => email;
            set => SetPropertyValue(nameof(Email), ref email, value);
        }

        string religion;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Religion
        {
            get => religion;
            set => SetPropertyValue(nameof(Religion), ref religion, value);
        }

        DateTime employmentDate;
        public DateTime EmploymentDate
        {
            get => employmentDate;
            set => SetPropertyValue(nameof(EmploymentDate), ref employmentDate, value);
        }

        string socialMediaAccountName;
        [DevExpress.Xpo.DisplayName("Social Media Account")]
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string SocialMediaAccountName
        {
            get => socialMediaAccountName;
            set => SetPropertyValue(nameof(SocialMediaAccountName), ref socialMediaAccountName, value);
        }

        private Image employeePhoto;
        [ImmediatePostData]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [ValueConverter(typeof(ImageValueConverter))]
        [FileTypeFilter("Image files", 2, "*.png", "*.jpg")]
        [Appearance("Appearance:Employee.EmployeePhoto.UserCanEdit", Enabled = false, Criteria = "!UserCanEdit", Context = "DetailView")]
        [ImageEditor(ListViewImageEditorMode = ImageEditorMode.DropDownPictureEdit, DetailViewImageEditorMode = ImageEditorMode.DropDownPictureEdit)]
        public Image EmployeePhoto
        {
            get { return this.employeePhoto; }
            set { this.SetPropertyValue<Image>("EmployeePhoto", ref this.employeePhoto, value); }
        }

        //School school;
        //[Association("School-Employees")]
        //public School School
        //{
        //    get => school;
        //    set => SetPropertyValue(nameof(School), ref school, value);
        //}


    }

}