﻿using System;
using System.ComponentModel;
using System.Drawing;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.ConditionalAppearance;
using DevExpress.ExpressApp.DC;
using DevExpress.ExpressApp.Security.Strategy;
using DevExpress.Persistent.Base;
using DevExpress.Persistent.Validation;
using DevExpress.Xpo;
using DevExpress.Xpo.Metadata;
using K12MIS.Module.BusinessObjects;
using K12MIS.Module.BusinessObjects.Enums;

namespace K12MIS.Module.BusinessObjects
{
    [DefaultClassOptions]
    [DefaultProperty("StudentName")]
    //[DefaultListViewOptions(MasterDetailMode.ListViewOnly, false, NewItemRowPosition.None)]
    //[DefaultListViewOptions(MasterDetailMode.ListViewOnly, false, NewItemRowPosition.None)]
    //[ImageName("BO_Contact")]
    //[DefaultProperty("DisplayMemberNameForLookupEditorsOfThisType")]
    //[DefaultListViewOptions(MasterDetailMode.ListViewOnly, false, NewItemRowPosition.None)]
    //[Persistent("DatabaseTableName")]
    // Specify more UI options using a declarative approach (https://documentation.devexpress.com/#eXpressAppFramework/CustomDocument112701).
    public class Student : SecuritySystemUser
    { // Inherit from a different class to provide a custom primary key, concurrency and deletion behavior, etc. (https://documentation.devexpress.com/eXpressAppFramework/CustomDocument113146.aspx).
        public Student(Session session) : base(session)
        {
        }
        public override void AfterConstruction()
        {
            base.AfterConstruction();
            // Place your initialization code here (https://documentation.devexpress.com/eXpressAppFramework/CustomDocument112834.aspx).
        }
        //private string _PersistentProperty;
        //[XafDisplayName("My display name"), ToolTip("My hint message")]
        //[ModelDefault("EditMask", "(000)-00"), Index(0), VisibleInListView(false)]
        //[Persistent("DatabaseColumnName"), RuleRequiredField(DefaultContexts.Save)]
        //public string PersistentProperty {
        //    get { return _PersistentProperty; }
        //    set { SetPropertyValue("PersistentProperty", ref _PersistentProperty, value); }
        //}

        //[Action(Caption = "My UI Action", ConfirmationMessage = "Are you sure?", ImageName = "Attention", AutoCommit = true)]
        //public void ActionMethod() {
        //    // Trigger a custom business logic for the current record in the UI (https://documentation.devexpress.com/eXpressAppFramework/CustomDocument112619.aspx).
        //    this.PersistentProperty = "Paid";
        //}
        decimal currentBalance;
        Guardian guardian;
        string iDNumber;


        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        [Index(0), VisibleInListView(true), VisibleInDetailView(true)]
        [RuleRequiredField(DefaultContexts.Save)]
        [XafDisplayName("ID Number"), ToolTip("Enter the Student ID Number")]
        public string IDNumber
        {
            get => iDNumber;
            set => SetPropertyValue(nameof(IDNumber), ref iDNumber, value);
        }

        
        string lastName;
        [Index(1), VisibleInListView(true), VisibleInDetailView(true)]
        [RuleRequiredField(DefaultContexts.Save)]
        [XafDisplayName("Last Name"), ToolTip("Enter the Student Last Name")]
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string LastName
        {
            get => lastName;
            set => SetPropertyValue(nameof(LastName), ref lastName, value);
        }
        string firstName;
        [Index(2), VisibleInListView(true), VisibleInDetailView(true)]
        [RuleRequiredField(DefaultContexts.Save)]
        [XafDisplayName("First Name"), ToolTip("Enter the Student First Name")]
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string FirstName
        {
            get => firstName;
            set => SetPropertyValue(nameof(FirstName), ref firstName, value);
        }
        string middleName;
        [Index(3), VisibleInListView(true), VisibleInDetailView(true)]
        [RuleRequiredField(DefaultContexts.Save)]
        [XafDisplayName("Middle Name"), ToolTip("Enter the Student Middle Name")]
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string MiddleName
        {
            get => middleName;
            set => SetPropertyValue(nameof(MiddleName), ref middleName, value);
        }

        [PersistentAlias("concat(LastName, ' ', FirstName)")]
        public string StudentName
        {
            get
            {
                return Convert.ToString(EvaluateAlias("StudentName"));
            }
        }


        DateTime birthDate;
        public DateTime BirthDate
        {
            get => birthDate;
            set => SetPropertyValue(nameof(BirthDate), ref birthDate, value);
        }
        [NonPersistent]
        public int Age
        {
            get
            {
                if (BirthDate == DateTime.MinValue)
                    return 0;

                return (int)(DateTime.Today.Subtract(BirthDate).TotalDays / 365);
            }
        }
        private Gender _Gender;
        public Gender Gender
        {
            get
            {
                return _Gender;
            }
            set
            {
                SetPropertyValue("Gender", ref _Gender, value);
            }
        }
        string presentAddress;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string PresentAddress
        {
            get => presentAddress;
            set => SetPropertyValue(nameof(PresentAddress), ref presentAddress, value);
        }
        string permanentAddress;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string PermanentAddress
        {
            get => permanentAddress;
            set => SetPropertyValue(nameof(PermanentAddress), ref permanentAddress, value);
        }
        string email;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Email
        {
            get => email;
            set => SetPropertyValue(nameof(Email), ref email, value);
        }
        string religion;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Religion
        {
            get => religion;
            set => SetPropertyValue(nameof(Religion), ref religion, value);
        }
        string contactNo;
        [DevExpress.Xpo.DisplayName("Contact #")]
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string ContactNo
        {
            get => contactNo;
            set => SetPropertyValue(nameof(ContactNo), ref contactNo, value);
        }

        string bloodType;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string BloodType
        {
            get => bloodType;
            set => SetPropertyValue(nameof(BloodType), ref bloodType, value);
        }
        string height;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Height
        {
            get => height;
            set => SetPropertyValue(nameof(Height), ref height, value);
        }
        string weight;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string Weight
        {
            get => weight;
            set => SetPropertyValue(nameof(Weight), ref weight, value);
        }
        string emergencyContact;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string EmergencyContact
        {
            get => emergencyContact;
            set => SetPropertyValue(nameof(EmergencyContact), ref emergencyContact, value);
        }

        string emergencyContactNo;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string EmergencyContactNo
        {
            get => emergencyContactNo;
            set => SetPropertyValue(nameof(EmergencyContactNo), ref emergencyContactNo, value);
        }
        private Image studentPhoto;
        [ImmediatePostData]
        [VisibleInListView(false)]
        [VisibleInLookupListView(false)]
        [ValueConverter(typeof(ImageValueConverter))]
        [FileTypeFilter("Image files", 2, "*.png", "*.jpg")]
        //[Appearance("Appearance:Student.StudentPhoto.UserCanEdit", Enabled = false, Criteria = "!UserCanEdit", Context = "DetailView")]
        [ImageEditor(ListViewImageEditorMode = ImageEditorMode.DropDownPictureEdit, DetailViewImageEditorMode = ImageEditorMode.DropDownPictureEdit)]
        public Image StudentPhoto
        {
            get { return this.studentPhoto; }
            set { this.SetPropertyValue<Image>("StudentPhoto", ref this.studentPhoto, value); }
        }

        //[FieldSize(SizeAttribute.Unlimited), VisibleInListView(true)]
        //[VisibleInLookupListView(false)]
        //[ImageEditor(ListViewImageEditorMode = ImageEditorMode.PictureEdit,
        //  DetailViewImageEditorMode = ImageEditorMode.PictureEdit, ListViewImageEditorCustomHeight = 25)]
        //public byte[] Photo { get; set; }


        string specialNote;
        [Size(SizeAttribute.DefaultStringMappingFieldSize)]
        public string SpecialNote
        {
            get => specialNote;
            set => SetPropertyValue(nameof(SpecialNote), ref specialNote, value);
        }
        
        public decimal CurrentBalance
        {
            get => currentBalance;
            set => SetPropertyValue(nameof(CurrentBalance), ref currentBalance, value);
        }

        [Association("Student-Payments")]
        public XPCollection<Payment> Payments
        {
            get
            {
                return GetCollection<Payment>(nameof(Payments));
            }
        }
        
        [Association("Guardian-Students")]
        public Guardian Guardian
        {
            get => guardian;
            set => SetPropertyValue(nameof(Guardian), ref guardian, value);
        }

        [Association("GradeSection-Students")]
        public XPCollection<GradeSection> GradeSections
        {
            get
            {
                return GetCollection<GradeSection>(nameof(GradeSections));
            }
        }

    }
}